/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.components.java;

import com.zfabrik.util.runtime.Foundation;

/**
 * Supported language level abstraction
 */
public enum LangLevel { 
	/**
	 * Java 6 as of JDK 1.6.x
	 */
	JAVA6("1.6") , 
	
	/**
	 * Java 7 as of JDK 1.7.x
	 */
	JAVA7("1.7"),
	
	/**
	 * Java 8 as of JDK 1.8.x
	 */
	JAVA8("1.8"), 

	/**
	 * Java 9 as of JDK 9.x
	 */
	JAVA9("9"),

	/**
	 * Java 10 as of JDK 10.x
	 */
	JAVA10("10"),
	/**
	 * Java 11 as of JDK 11.x
	 */
	JAVA11("11"), 
	/**
	 * Java 12 as of JDK 12.x
	 */
	JAVA12("12"),
	/**
	 * Java 13 as of JDK 13.x
	 */
	JAVA13("13"); 

	public static LangLevel HIGHEST = JAVA13;
	
	private String value;

	LangLevel(String value) {
        this.value = value;
    }
    
    @Override
    public String toString() {
    	return value;
    }
    
    /**
	 * The current language level is taken from a system property or by inspecting the runtime. This method implements
	 * the official way. The current default is {@value #HIGHEST}.
	 */
	public static LangLevel determine() {
		String ls = System.getProperty(Foundation.LANGUAGE_LEVEL,"").trim();
		if (ls.length()>0) {
			LangLevel ll = parse(ls);
			if (ll==null) {
				throw new IllegalStateException("Unsupported system property value: "+Foundation.LANGUAGE_LEVEL+"="+ls);
			}
			return ll;
		}
		String vs = System.getProperty("java.version");
		if (vs!=null) {
			if (vs.startsWith("13")) {
				return JAVA13;
			} 
			if (vs.startsWith("12")) {
				return JAVA12;
			} 
			if (vs.startsWith("11")) {
				return JAVA11;
			} 
			if (vs.startsWith("10")) {
				return JAVA10;
			} 
			if (vs.startsWith("9")) {
				return JAVA9;
			} 
			if (vs.startsWith("1.8")) {
				return JAVA8;
			} 
			if (vs.startsWith("1.7")) {
				return JAVA7;
			} 
			if (vs.startsWith("1.6")) {
				return JAVA6;
			} 
		}
		return HIGHEST;
	}

	public static LangLevel parse(String value) {
    	for (LangLevel l : LangLevel.values()) {
    		if (l.value.equals(value)) {
    			return l;
    		}
    	}
    	return null;
    }

}