/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.dev.transport;

import java.util.Comparator;
import java.util.Set;
import java.util.TreeSet;

public class ProjectDelta {
	public final static char NEW = '?'; 
	public final static char MOD = 'M'; 
	public final static char ADD = 'A'; 
	public final static char DEL = 'D'; 
	
	private char type = MOD;
	private String name;
	
	// use reverse order so that the maximum revision number will be the first entry
	private Set<Long> revs = new TreeSet<Long>(new Comparator<Long>() {
        public int compare(Long o1, Long o2) {
            return (int) (o2-o1);
        }
    });
	
	public ProjectDelta(String name) {
		this.name = name;
	}
	
	public void setType(char type) {
		this.type =type;
	}
	
	public char getType() {
		return type;
	}	

	public String getName() {
		return name;
	}
	
	
	public Set<Long> getRevs() {
		return revs;
	}
	
	public Long getMaxRev() {
	    Long result = revs.isEmpty()? -1L : revs.iterator().next();
	    return result;
	}
	
	@Override
	public int hashCode() {
		return this.name.hashCode();
	}
	
	
	
	@Override
	public boolean equals(Object obj) {
		if (this==obj) 
			return true;
		if (obj instanceof ProjectDelta)
			return this.name.equals(((ProjectDelta)obj).name);
		return false;
	}
	
	public String toString() {
		return name+" ["+type+", maxRev: " + getMaxRev() + "]";
	}
}
