/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.dev.transport.gui;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

public class RevisionChooser extends JPanel implements ActionListener {
	private static final long serialVersionUID = 1L;
	private JTextField sw_startRevision, sw_endRevision, sw_targetRevision;
	private ActionListener parent;
	
	public RevisionChooser(ActionListener parent) {
		this.parent = parent;
		setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Revision Range"),
                BorderFactory.createEmptyBorder(5,5,5,5)));

		GridBagLayout grid = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(5,2,5,2);
		setLayout(grid);

		c.gridwidth=1;
		c.weightx=0;
		JLabel l = _makeLabel("Source base");
		l.setToolTipText("This is the revision to start delta computation from. Think about it as a source level you already have in your target repository");
		_addWithConstraints(grid,c, l);

		c.weightx=1;
		this.sw_startRevision = new JTextField(4);
		_addWithConstraints(grid,c, this.sw_startRevision);
		
		c.weightx=0;
		c.gridwidth=1;
		JButton bu = new JButton("...");
		bu.setActionCommand(TransportTool.AC_PICK_STARTREV);
		bu.addActionListener(this);
		_addWithConstraints(grid,c, bu);

		c.weightx=0;
		l = _makeLabel("Source end");
		l.setToolTipText("This is the revision to end delta computation at. Think about it as a source level you want to achieve your target repository");
		_addWithConstraints(grid,c, l);

		c.weightx=1;
		this.sw_endRevision = new JTextField(4);
		this.sw_endRevision.setText("HEAD");
		_addWithConstraints(grid,c, this.sw_endRevision);

		c.weightx=0;
		c.gridwidth=1;
		bu = new JButton("...");
		bu.setActionCommand(TransportTool.AC_PICK_ENDREV);
		bu.addActionListener(this);
		_addWithConstraints(grid,c, bu);

		l = _makeLabel("Target rev");
		l.setToolTipText("This is the revision of the target repository you want to work on when applying the delta");
		c.weightx=0;
		_addWithConstraints(grid,c, l);

		c.weightx=1;
		this.sw_targetRevision = new JTextField(4);
		this.sw_targetRevision.setText("HEAD");
		_addWithConstraints(grid,c, this.sw_targetRevision);
		
		c.weightx=0;
		bu = new JButton("...");
		bu.setActionCommand(TransportTool.AC_PICK_TARGETREV);
		bu.addActionListener(this);
		_addWithConstraints(grid,c, bu);

		c.weightx=0.3;
		c.gridwidth=GridBagConstraints.REMAINDER;
		_addWithConstraints(grid, c, new JLabel());
	}

	private JLabel _makeLabel(String title) {
		JLabel l=new JLabel(title);
		l.setHorizontalAlignment(SwingConstants.RIGHT);
		return l;
	}

	private void _addWithConstraints(GridBagLayout grid,
			GridBagConstraints c, Component comp) {
		grid.setConstraints(comp, c);
		add(comp);
	}

	public void actionPerformed(ActionEvent e) {
		String actionCommand = e.getActionCommand();
		if (TransportTool.AC_PICK_ENDREV.equals(actionCommand)) {
			this.parent.actionPerformed(e);
		} else
		if (TransportTool.AC_PICK_STARTREV.equals(actionCommand)) {
			this.parent.actionPerformed(e);
		} else
		if (TransportTool.AC_PICK_TARGETREV.equals(actionCommand)) {
			this.parent.actionPerformed(e);
		}
	}
	
	public void setStartRev(String rev) {
		this.sw_startRevision.setText(rev);
	}
	
	public void setEndRev(String rev) {
		this.sw_endRevision.setText(rev);
	}

	public void setTargetRev(String rev) {
		this.sw_targetRevision.setText(rev);
	}
	
	public String getStartRev() {
		return this.sw_startRevision.getText();
	}
	
	public String getEndRev() {
		return this.sw_endRevision.getText();
	}

	public String getTargetRev() {
		return this.sw_targetRevision.getText();
	}

}
