/*
 * z2-Environment
 * 
 * Copyright(c) ZFabrik Software GmbH & Co. KG
 * 
 * contact@zfabrik.de
 * 
 * http://www.z2-environment.eu
 */
package com.zfabrik.impl.dev.transport.gui;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

public class WorkSpaceChooser extends JPanel implements ActionListener {
	private static final long serialVersionUID = 1L;
	private static final String AC_PICK = "PI";
	private JTextField sw_workspaceFolder;
	private JFileChooser fileChooser;
	private ActionListener parent;
	
	public WorkSpaceChooser(ActionListener parent) {
		this.parent = parent;
		
		setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Target Workspace"),
                BorderFactory.createEmptyBorder(5,5,5,5)));

		GridBagLayout grid = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(5,2,5,2);
		setLayout(grid);

		c.gridwidth=1;
		c.weightx=0;
		_addWithConstraints(grid,c, _makeLabel("Root Folder"));

		c.weightx=1;
		this.sw_workspaceFolder = new JTextField();
		_addWithConstraints(grid,c, this.sw_workspaceFolder);
		
		c.weightx=0;
		c.gridwidth=1;
		JButton bu = new JButton("...");
		bu.setActionCommand(AC_PICK);
		bu.addActionListener(this);
		_addWithConstraints(grid,c, bu);

		c.weightx=0;
		c.gridwidth=1;
		bu = new JButton("Generate Now!");
		bu.setActionCommand(TransportTool.AC_RUN);
		bu.addActionListener(this);
		_addWithConstraints(grid,c, bu);
		
		c.weightx=0.3;
		c.gridwidth=GridBagConstraints.REMAINDER;
		_addWithConstraints(grid,c, new JLabel());

		this.fileChooser = new JFileChooser();
		this.fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	}

	private JLabel _makeLabel(String title) {
		JLabel l=new JLabel(title);
		l.setHorizontalAlignment(SwingConstants.RIGHT);
		return l;
	}

	private void _addWithConstraints(GridBagLayout grid,
			GridBagConstraints c, Component comp) {
		grid.setConstraints(comp, c);
		add(comp);
	}

	public void actionPerformed(ActionEvent e) {
		String actionCommand = e.getActionCommand();
		if (AC_PICK.equals(actionCommand)) {
	        int returnVal = this.fileChooser.showOpenDialog(this);

	        if (returnVal == JFileChooser.APPROVE_OPTION) {
	        	this.sw_workspaceFolder.setText(this.fileChooser.getSelectedFile().toString());
	        }
		} else
		if (TransportTool.AC_RUN.equals(actionCommand)) {
			this.parent.actionPerformed(e);
		}
	}
	
	public File getWorkspaceRoot() {
		String f = this.sw_workspaceFolder.getText().trim();
		if (f.length()==0) return null;
		return new File(f);
	}
	
}
